/* /////////////////////////////////////////////////////////////////////////////
 * File:        reclspp_fileentry.h
 *
 * Purpose:     recls C++ mapping - FileEntry class.
 *
 * Created:     18th August 2003
 * Updated:     10th November 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *                          http://www.recls.org/
 *
 *              email:      submissions@recls.org  for submissions
 *                          admin@recls.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLSPP_FILEENTRY
#define RECLS_INCL_H_RECLSPP_FILEENTRY

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLSPP_FILEENTRY_MAJOR    1
# define RECLS_VER_H_RECLSPP_FILEENTRY_MINOR    3
# define RECLS_VER_H_RECLSPP_FILEENTRY_REVISION 3
# define RECLS_VER_H_RECLSPP_FILEENTRY_EDIT     16
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file reclspp_fileentry.h Definition of the \link recls::cpp::FileEntry FileEntry\endlink and
 * \link recls::cpp::DirectoryParts DirectoryParts\endlink classes, part of the 
 * \ref group_recls_cpp "recls C++ mapping" */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "reclspp.h"
#include <recls_assert.h>
#include <stlsoft_operator_bool.h>	// For operator bool() stuff

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace cpp
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

class FileEntry;
class FileSearch;

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// This class provides enumeration over the parts of a directory for a given file entry
///
/// \ingroup group_recls_cpp
class DirectoryParts
{
public:
    typedef recls_char_t    char_type;
    typedef DirectoryParts  class_type;

// Construction
private:
    friend class FileEntry;
    DirectoryParts(recls_info_t info);

/// \name Construction
/// @{
public:
    /// Default constructor
    DirectoryParts();
    /// Copy constructor
    DirectoryParts(DirectoryParts const &rhs);
    /// Destructor
    ~DirectoryParts();

    /// Copy assignment operator
    DirectoryParts &operator =(DirectoryParts const &rhs);

/// @}

/// \name Attributes
/// @{
public:
    /// Returns the number of directory parts managed by this instance
    size_t      size() const;
    /// Returns the number of directory parts managed by this instance
    ///
    /// \param index The index of the requried directory part
    ///
    /// \note The behaviour is undefined if index is out of the range denoted by \c [0, size())
    string_t    operator [](size_t index) const;

/// @}

// Implementation
private:
    static recls_info_t _copy_info(recls_info_t info);

// Members
private:
    recls_info_t    m_info;
};

/// This class represents a file search entry
///
/// \ingroup group_recls_cpp
class FileEntry
{
public:
    typedef recls_char_t    char_type;
    typedef FileEntry       class_type;

// Construction
private:
    friend class FileSearch;
    FileEntry(recls_info_t info);

/// \name Construction
/// @{
public:
    /// Default constructor
    FileEntry();
    /// Copy constructor
    FileEntry(FileEntry const &rhs);
    /// Destructor
    ~FileEntry();

    /// Copy assignment operator
    FileEntry &operator =(FileEntry const &rhs);

/// @}

/// \name State
/// @{
private:
	typedef stlsoft_ns_qual(operator_bool_generator)<class_type>::class_type	boolean_type;
public:
    /// Indicates the state of the instance, i.e. whether it is contains a valid recls_info_t or not
	operator boolean_type::return_type() const
	{
		return boolean_type::translate(NULL != m_info);
	}

#ifndef STLSOFT_CF_OPERATOR_NOT_VIA_OPERATOR_POINTER_TO_MEMBER_SUPPORT
    /// Returns the logical inverse of the state of the instance, i.e. whether it is contains a valid recls_info_t or not
    bool operator !() const;
#endif /* !STLSOFT_CF_OPERATOR_NOT_VIA_OPERATOR_POINTER_TO_MEMBER_SUPPORT */

/// @}

/// \name Attributes
/// @{
public:
    /// Returns a pointer to the full path of the file entry, or the empty string
    char_type const     *c_str() const;

    /// Returns the full path of the file entry
    string_t            GetPath() const;
#ifdef RECLS_PLATFORM_API_WIN32
    /// Returns the drive of the file entry
    char_type           GetDrive() const;
#endif /* RECLS_PLATFORM_API_WIN32 */
    /// Returns the directory of the file entry
    string_t            GetDirectory() const;
    /// Returns the directory (+ drive) of the file entry
    string_t            GetDirectoryPath() const;
    /// Returns the directory parts of the file entry
    DirectoryParts      GetDirectoryParts() const;
    /// Returns the file (filename + extension) of the file entry
    string_t            GetFile() const;
    /// Returns the short form of the file (filename + extension) of the file entry
    ///
    /// \Note On systems other than Win32, this is identical to GetFile()
    string_t            GetShortFile() const;
    /// Returns the filename of the file entry
    string_t            GetFileName() const;
    /// Returns the file extension of the file entry
    string_t            GetFileExt() const;

    /// Returns the creation time of the file entry
    ///
    /// \Note On systems other than Win32, this is identical to GetModificationTime()
    recls_time_t        GetCreationTime() const;
    /// Returns the modification time of the file entry
    recls_time_t        GetModificationTime() const;
    /// Returns the last access time of the file entry
    recls_time_t        GetLastAccessTime() const;
    /// Returns the last statuc change time of the file entry
    ///
    /// \Note On systems other than UNIX, this is identical to GetModificationTime()
    recls_time_t        GetLastStatusChangeTime() const;

    /// Returns the size of the file entry
    recls_filesize_t    GetSize() const;

    /// Indicates whether the file entry is read-only
    recls_bool_t        IsReadOnly() const;
    /// Indicates whether the file entry is a directory
    recls_bool_t        IsDirectory() const;
    /// Indicates whether the file entry is a link
    recls_bool_t        IsLink() const;

/// @}

// Implementation
private:
    static recls_info_t _copy_info(recls_info_t info);

// Members
private:
    recls_info_t    m_info;
};

/* /////////////////////////////////////////////////////////////////////////////
 * Shims
 */

/// String access shim
/// \ingroup group_recls_cpp
///
/// This returns a non-null null-terminated C-string of the given FileEntry's full path.
/// \param fe The FileEntry instance whose path is to be returned
///
/// \note This fits the <a href = "http://stlsoft.org/help/main.html">STLSoft</a> shims 
/// concept, described in <a href = "http://synesis.com.au/resources/articles/cpp/shims.pdf">this</a>
/// Synesis Software White Paper, and featured in the article "<a href = "http://www.cuj.com/documents/s=8681/cuj0308wilson/">Generalised String Manipulation: Access Shims and Type-tunnelling</a>",
/// in the August 2003 issue of <a href = "http://www.cuj.com">C/C++ User's Journal</a>.
inline FileEntry::char_type const *c_str_ptr(FileEntry const &fe)
{
    return fe.c_str();
}

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

inline /* static */ recls_info_t DirectoryParts::_copy_info(recls_info_t info)
{
    recls_info_t    newInfo;

    if( NULL == info ||
        RECLS_FAILED(Recls_CopyDetails(info, &newInfo)))
    {
        newInfo = NULL;
    }

    return newInfo;
}

inline DirectoryParts::DirectoryParts(recls_info_t info)
    : m_info(_copy_info(info))
{}

inline DirectoryParts::DirectoryParts()
    : m_info(NULL)
{}

inline DirectoryParts::DirectoryParts(DirectoryParts const &rhs)
    : m_info(_copy_info(rhs.m_info))
{}

inline DirectoryParts::~DirectoryParts()
{
    Recls_CloseDetails(m_info);
}

inline DirectoryParts &DirectoryParts::operator =(DirectoryParts const &rhs)
{
    Recls_CloseDetails(m_info);

    m_info = _copy_info(rhs.m_info);

    return *this;
}

inline size_t DirectoryParts::size() const
{
    return Recls_GetDirectoryPartProperty(m_info, -1, NULL, 0);
}

inline string_t DirectoryParts::operator [](size_t index) const
{
    char_type dirPart[RECLS_PATH_MAX];

    recls_assert(NULL != m_info);
    recls_assert(index < size());

    return RECLS_SUCCEEDED(Recls_GetDirectoryPartProperty(m_info, index, dirPart, RECLS_PATH_MAX)) ? string_t(dirPart) : string_t();
}

inline /* static */ recls_info_t FileEntry::_copy_info(recls_info_t info)
{
    recls_info_t    newInfo;

    if( NULL == info ||
        RECLS_FAILED(Recls_CopyDetails(info, &newInfo)))
    {
        newInfo = NULL;
    }

    return newInfo;
}

inline FileEntry::FileEntry(recls_info_t info)
    : m_info(info)
{}

inline FileEntry::FileEntry()
    : m_info(NULL)
{}

inline FileEntry::FileEntry(FileEntry const &rhs)
    : m_info(_copy_info(rhs.m_info))
{}

inline FileEntry::~FileEntry()
{
	if(NULL != m_info)
	{
		Recls_CloseDetails(m_info);
	}
}

inline FileEntry &FileEntry::operator =(FileEntry const &rhs)
{
    Recls_CloseDetails(m_info);

    m_info = _copy_info(rhs.m_info);

    return *this;
}

#ifndef STLSOFT_CF_OPERATOR_NOT_VIA_OPERATOR_POINTER_TO_MEMBER_SUPPORT
inline bool FileEntry::operator !() const
{
    return (*this) ? false : true;
}
#endif /* !STLSOFT_CF_OPERATOR_NOT_VIA_OPERATOR_POINTER_TO_MEMBER_SUPPORT */


inline FileEntry::char_type const *FileEntry::c_str() const
{
    return (NULL != m_info) ? m_info->path.begin : "";
}

inline string_t FileEntry::GetPath() const
{
    recls_assert(NULL != m_info);

    return string_t(m_info->path.begin, m_info->path.end);
}

#ifdef RECLS_PLATFORM_API_WIN32
inline FileEntry::char_type FileEntry::GetDrive() const
{
    char_type chDrive;

    return (Recls_GetDriveProperty(m_info, &chDrive), chDrive);
}
#endif /* RECLS_PLATFORM_API_WIN32 */

inline string_t FileEntry::GetDirectory() const
{
    recls_assert(NULL != m_info);

    return string_t(m_info->directory.begin, m_info->directory.end);
}

inline string_t FileEntry::GetDirectoryPath() const
{
    recls_assert(NULL != m_info);

    return string_t(m_info->path.begin, m_info->directory.end);
}

inline DirectoryParts FileEntry::GetDirectoryParts() const
{
    recls_assert(NULL != m_info);

    return DirectoryParts(m_info);
}

inline string_t FileEntry::GetFile() const
{
    recls_assert(NULL != m_info);
    recls_assert(m_info->fileName.begin <= m_info->fileExt.end);

    return string_t(m_info->fileName.begin, m_info->fileExt.end);
}

inline string_t FileEntry::GetShortFile() const
{
#ifdef RECLS_PLATFORM_API_WIN32
    recls_assert(NULL != m_info);

    return string_t(m_info->shortFile.begin, m_info->shortFile.end);
#else
    return GetFile();
#endif /* RECLS_PLATFORM_API_WIN32 */
}

inline string_t FileEntry::GetFileName() const
{
    recls_assert(NULL != m_info);

    return string_t(m_info->fileName.begin, m_info->fileName.end);
}

inline string_t FileEntry::GetFileExt() const
{
    recls_assert(NULL != m_info);

    return string_t(m_info->fileExt.begin, m_info->fileExt.end);
}

inline recls_time_t FileEntry::GetCreationTime() const
{
    recls_assert(NULL != m_info);

    return Recls_GetCreationTime(m_info);
}

inline recls_time_t FileEntry::GetModificationTime() const
{
    recls_assert(NULL != m_info);

    return Recls_GetModificationTime(m_info);
}

inline recls_time_t FileEntry::GetLastAccessTime() const
{
    recls_assert(NULL != m_info);

    return Recls_GetLastAccessTime(m_info);
}

inline recls_time_t FileEntry::GetLastStatusChangeTime() const
{
    recls_assert(NULL != m_info);

    return Recls_GetLastStatusChangeTime(m_info);
}

inline recls_filesize_t FileEntry::GetSize() const
{
    recls_assert(NULL != m_info);

    return m_info->size;
}

inline recls_bool_t FileEntry::IsReadOnly() const
{
    recls_assert(NULL != m_info);

    return Recls_IsFileReadOnly(m_info);
}

inline recls_bool_t FileEntry::IsDirectory() const
{
    recls_assert(NULL != m_info);

    return Recls_IsFileDirectory(m_info);
}

inline recls_bool_t FileEntry::IsLink() const
{
    recls_assert(NULL != m_info);

    return Recls_IsFileLink(m_info);
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace cpp */
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLSPP_FILEENTRY */

/* ////////////////////////////////////////////////////////////////////////// */
