/* /////////////////////////////////////////////////////////////////////////////
 * File:        reclspp_filesearch.h
 *
 * Purpose:     recls C++ mapping - FileSearch class.
 *
 * Created:     18th August 2003
 * Updated:     2nd November 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *                          http://www.recls.org/
 *
 *              email:      submissions@recls.org  for submissions
 *                          admin@recls.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_H_RECLSPP_FILESEARCH
#define RECLS_INCL_H_RECLSPP_FILESEARCH

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_H_RECLSPP_FILESEARCH_MAJOR       1
# define RECLS_VER_H_RECLSPP_FILESEARCH_MINOR       1
# define RECLS_VER_H_RECLSPP_FILESEARCH_REVISION    1
# define RECLS_VER_H_RECLSPP_FILESEARCH_EDIT        10
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file reclspp_filesearch.h Definition of the \link recls::cpp::FileSearch FileSearch\endlink class, part of 
 * the \ref group_recls_cpp "recls C++ mapping"
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "reclspp.h"
#include "reclspp_fileentry.h"
#include <recls_assert.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace cpp
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// This class provides functionality for executing a file search
///
/// \ingroup group_recls_cpp
class FileSearch
{
public:

/// \name Construction
/// @{
public:
    /// Creates a search for the items identified by \c pattern in the directory \c rootDir, according to the given \c flags.
    ///
    /// \param rootDir The starting directory for the search. If NULL, or the empty string, uses the current directory
    /// \param pattern The search pattern, e.g. "*.h". If NULL, or the empty string, uses "*.*" (Win32) or "*" (UNIX)
    /// \param flags A combination of enumerants from the \c RECLS_FLAG enumeration
    FileSearch(recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags);
    /// Closes the search and releases resources
    ~FileSearch();

/// @}

/// \name Operations
/// @{
public:
    /// Advances the search to the next position
    recls_rc_t      GetNext();

/// @}

/// \name Attributes
/// @{
public:
    /// Returns non-zero if there is more data available
    recls_bool_t    HasMoreElements() const;
    /// Returns the current entry
    ///
    /// \note The behaviour is undefined when the \c HasMoreElements() returns zero
    FileEntry       GetCurrentEntry() const;

    /// Returns the last error status code for this search object
    recls_rc_t      GetLastError() const;

    /// Returns the number of outstanding file-entry blocks associated with this search object
    recls_uint32_t  GetNumOutstandingDetails() const;

/// @}

// Implementation
private:

// Members
private:
    hrecls_t /* const */    m_hSrch;
    mutable recls_rc_t      m_lastError;

// Not to be implemented
private:
    FileSearch(FileSearch const &rhs);
    FileSearch &operator =(FileSearch const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

inline FileSearch::FileSearch(recls_char_t const *rootDir, recls_char_t const *pattern, recls_uint32_t flags)
    : /* m_hSrch(NULL) // This must not be initialised in the constructor
    , */ m_lastError((m_hSrch = NULL, Recls_Search(rootDir, pattern, flags, &const_cast<hrecls_t&>(m_hSrch))))
{}

inline FileSearch::~FileSearch()
{
    if(NULL != m_hSrch)
    {
        Recls_SearchClose(m_hSrch);
    }
}

inline recls_rc_t FileSearch::GetNext()
{
    recls_assert(NULL != m_hSrch);

    recls_rc_t  rc  =   Recls_GetNext(m_hSrch);

    if(RECLS_FAILED(rc))
    {
        Recls_SearchClose(m_hSrch);
        m_hSrch = NULL;
    }

    m_lastError = rc;

    return rc;
}

inline recls_bool_t FileSearch::HasMoreElements() const
{
    return RECLS_SUCCEEDED(m_lastError);
}

inline FileEntry FileSearch::GetCurrentEntry() const
{
    recls_info_t    info;
    recls_rc_t      rc  =   Recls_GetDetails(m_hSrch, &info);

    return RECLS_SUCCEEDED(m_lastError = rc) ? FileEntry(info) : FileEntry();
}

inline recls_rc_t FileSearch::GetLastError() const
{
    return m_lastError;
}

inline recls_uint32_t FileSearch::GetNumOutstandingDetails() const
{
    recls_uint32_t  cDetails;
    recls_rc_t      rc  =   Recls_OutstandingDetails(m_hSrch, &cDetails);

    if(RECLS_FAILED(m_lastError = rc))
    {
        cDetails = 0;
    }

    return cDetails;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace cpp */
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_H_RECLSPP_FILESEARCH */

/* ////////////////////////////////////////////////////////////////////////// */
