/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_Search.cpp
 *
 * Purpose:     JNI implementation file for the Search class
 *
 * Created      21st November 2003
 * Updated:     24th November 2003
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:        www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <recls_Search.h>

#include <recls.h>
#if defined(RECLS_PLATFORM_IS_WIN32)
# include <recls_win32.h>
#elif defined(RECLS_PLATFORM_IS_UNIX)
# include <recls_unix.h>
#else
# error Platform not (yet) recognised
#endif /* platform */

#include <stlsoft.h>
#include <stlsoft_auto_buffer.h>
#include <stlsoft_malloc_allocator.h>

#include <stdarg.h>

#include "recls_jni_util.h"

/* ////////////////////////////////////////////////////////////////////////// */

#if !defined(RECLS_NO_NAMESPACE)
using recls::hrecls_t;
using recls::recls_info_t;
using recls::recls_strptrs_t;

using recls::Recls_Search;
using recls::Recls_SearchClose;
using recls::Recls_GetLastError;
using recls::Recls_GetDetails;
using recls::Recls_GetNext;
using recls::Recls_GetDirectoryPartProperty;
#endif /* !RECLS_NO_NAMESPACE */

typedef	stlsoft_ns_qual(auto_buffer)<recls_char_t, stlsoft_ns_qual(malloc_allocator)<recls_char_t>, RECLS_PATH_MAX>	buffer_t;

/* ////////////////////////////////////////////////////////////////////////// */

jfieldID GetSearchFieldID(JNIEnv *henv, jobject obj);
hrecls_t SearchFromJObject(JNIEnv *henv, jobject obj);

/* ////////////////////////////////////////////////////////////////////////// */


/* ////////////////////////////////////////////////////////////////////////// */


/*
 * Class:     org_recls_Search
 * Method:    Close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_org_recls_Search_Close
  (JNIEnv *henv, jobject obj)
{
	// Get the handle from the object
	hrecls_t	hSrch	=	SearchFromJObject(henv, obj);

	if(NULL == hSrch)
	{
//		ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Entry is already closed");
	}
	else
	{
		jfieldID	fid			=	GetSearchFieldID(henv, obj);

		henv->SetIntField(obj, fid, 0);

		Recls_SearchClose(hSrch);
	}
}

/*
 * Class:     org_recls_Search
 * Method:    Initialise
 * Signature: (Ljava/lang/String;Ljava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_org_recls_Search_Initialise
  (JNIEnv *henv, jobject obj, jstring jsSearchRoot, jstring jsPattern, jint flags)
{
	// Need to 
	recls_char_t const	*searchRoot	=	SafeStringUTFChars(henv, jsSearchRoot, NULL);
	recls_char_t const	*pattern	=	SafeStringUTFChars(henv, jsPattern, NULL);

	recls_char_t	searchRoot_[RECLS_PATH_MAX];
	recls_char_t	pattern_[RECLS_PATH_MAX];

	if(0 == *searchRoot)
	{
		searchRoot_[0] = '.';
		searchRoot_[1] = '\0';

		searchRoot = searchRoot_;
	}

	if(0 == *pattern)
	{
#if defined(RECLS_PLATFORM_IS_WIN32)
		pattern_[0] = '*';
		pattern_[1] = '.';
		pattern_[2] = '*';
		pattern_[3] = '\0';
#elif defined(RECLS_PLATFORM_IS_UNIX)
		pattern_[0] = '*';
		pattern_[1] = '\0';
#else
# error Platform not (yet) recognised
#endif /* platform */

		pattern = pattern_;
	}

	hrecls_t    hSrch;
	recls_rc_t	rc	=	Recls_Search(searchRoot, pattern, flags, &hSrch);

	if( RECLS_FAILED(rc) &&
		rc != RECLS_RC_NO_MORE_DATA)
	{
		ThrowReclsException(henv, "Search failed", rc);
	}

	return reinterpret_cast<jint>(hSrch);
}

/*
 * Class:     org_recls_Search
 * Method:    hasMoreElements_
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_recls_Search_hasMoreElements_1
  (JNIEnv *henv, jobject obj)
{
	hrecls_t	hSrch	=	SearchFromJObject(henv, obj);

	if(NULL == hSrch)
	{
		return JNI_FALSE;
	}
	else
	{
		recls_rc_t	rc	=	Recls_GetLastError(hSrch);

		return RECLS_SUCCEEDED(rc) ? JNI_TRUE : JNI_FALSE;
	}
}

/*
 * Class:     org_recls_Search
 * Method:    nextElement_
 * Signature: ()Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_org_recls_Search_nextElement_1
  (JNIEnv *henv, jobject obj)
{
	// This must return the current one, and then advance to the next
	hrecls_t	hSrch	=	SearchFromJObject(henv, obj);
	recls_rc_t	rc;
	jobject		entry;

	if(	NULL == hSrch ||
		RECLS_FAILED(rc = Recls_GetLastError(hSrch)))
	{
		ThrowException(henv, CLASSNAME_NOSUCHELEMENTX, "No more elements in search");

		entry = NULL;
	}
	else
	{
		recls_info_t	hEntry;

		rc = Recls_GetDetails(hSrch, &hEntry);

		if(RECLS_SUCCEEDED(rc))
		{
			jclass			clsEntry		=	henv->FindClass(CLASSNAME_ENTRY);
			jclass			clsString		=	henv->FindClass("Ljava/lang/String;");
			jmethodID		midEntryCtor	=	henv->GetMethodID(clsEntry, "<init>", "(ILjava/lang/String;[Ljava/lang/String;)V");
			size_t			cDirParts		=	Recls_GetDirectoryPartProperty(hEntry, -1, NULL, 0);
			jstring			path			=	StringFromRange(henv, hEntry->path.begin, hEntry->path.end);
			jobjectArray	dirParts		=	henv->NewObjectArray(cDirParts, clsString, NULL);

			for(size_t i = 0; i < cDirParts; ++i)
			{
				recls_strptrs_t const	*part	=	hEntry->directoryParts.begin + i;

				jstring dirPart = StringFromRange(henv, part->begin, part->end);

				henv->SetObjectArrayElement(dirParts, i, dirPart);
			}

			entry = henv->NewObject(clsEntry, midEntryCtor, hEntry, path, dirParts);

			// (ii) Now do a get-next, and if that returns NO_MORE_DATA, close the element
			rc = Recls_GetNext(hSrch);

			if(RECLS_FAILED(rc))
			{
				jfieldID	fid = GetSearchFieldID(henv, obj);

				henv->SetIntField(obj, fid, 0);

				Recls_SearchClose(hSrch);
			}
		}
		else
		{
			ThrowException(henv, CLASSNAME_ILLEGALSTATEX, "Search entry retrieval failed");
		}
	}

	return entry;	
}

/* ////////////////////////////////////////////////////////////////////////// */

jfieldID GetSearchFieldID(JNIEnv *henv, jobject obj)
{
	jclass		clsEntry	=	henv->GetObjectClass(obj);
	jfieldID	fid			=	henv->GetFieldID(clsEntry, FIELDNAME_HSRCH, "I");

//	henv->DeleteLocalRef(clsEntry);

	return fid;
}

hrecls_t SearchFromJObject(JNIEnv *henv, jobject obj)
{
	jfieldID	fid = GetSearchFieldID(henv, obj);

	return reinterpret_cast<hrecls_t>(henv->GetIntField(obj, fid));
}

/* ////////////////////////////////////////////////////////////////////////// */


